@@ -734,9 +734,9 @@ babl_space_from_icc (const char   *icc_data,
   const Babl *trc_blue  = NULL;
   const char *int_err;
   Babl *ret = NULL;
+  int speed_over_accuracy = intent & BABL_ICC_INTENT_PERFORMANCE;
 
-  sign_t profile_class, color_space;
-
+  sign_t profile_class, color_space, pcs;
 
   if (!error) error = &int_err;
   *error = NULL;
@@ -745,29 +745,41 @@ babl_space_from_icc (const char   *icc_data,
   {
     *error = "icc profile length inconsistency";
   }
-#if 0
-  else if (icc_ver_major > 2)
-  {
-    *error = "only ICC v2 profiles supported";
-  }
-#endif
   else
   {
-  profile_class = icc_read (sign, 12);
-  if (strcmp (profile_class.str, "mntr"))
-    *error = "not a monitor-class profile";
-  else
-  {
-  color_space = icc_read (sign, 16);
-  if (strcmp (color_space.str, "RGB "))
-    *error = "not defining an RGB space";
-  }
+    profile_class = icc_read (sign, 12);
+    if (strcmp (profile_class.str, "mntr"))
+      *error = "not a monitor-class profile";
+     else
+    {
+      color_space = icc_read (sign, 16);
+      if (strcmp (color_space.str, "RGB "))
+        *error = "not defining an RGB space";
+    }
   }
 
-  switch (intent)
+  if (!*error)
+  {
+    pcs = icc_read (sign, 20);
+    if (strcmp (pcs.str, "XYZ "))
+      *error = "PCS is not XYZ";
+  }
+
+  if (!*error)
+  switch (intent & 7) /* enum of intent is in lowest bits */
   {
     case BABL_ICC_INTENT_RELATIVE_COLORIMETRIC:
       /* that is what we do well */
+
+      if (!speed_over_accuracy)
+      {
+        if (icc_tag (state, "A2B0", NULL, NULL) &&
+            icc_tag (state, "B2A0", NULL, NULL))
+        {
+          *error = "use lcms, accuracy desired and cluts are present";
+        }
+      }
+
       break;
     case BABL_ICC_INTENT_PERCEPTUAL:
       /* if there is an A2B0 and B2A0 tags, we do not do what that
@@ -848,6 +860,19 @@ babl_space_from_icc (const char   *icc_data,
      wY = icc_read (s15f16, offset + 8 + 4);
      wZ = icc_read (s15f16, offset + 8 + 4 * 2);
 
+      /* detect inconsistent Argyll cLUT + matrix profiles */
+      if (icc_tag (state, "A2B0", NULL, NULL) ||
+          icc_tag (state, "B2A0", NULL, NULL))
+      {
+        if (rz > rx)
+        {
+           *error = "Inconsistent ICC profile detected, profile contains both cLUTs and a matrix with swapped primaries, this likely means it is an intentionally inconsistent Argyll profile is in use; this profile is only capable of high accuracy rendering and does not permit acceleration for interactive previews.";
+           fprintf (stderr, "babl ICC warning: %s\n", *error);
+           babl_free (state);
+           return NULL;
+        }
+      }
+
      ret = (void*)babl_space_match_trc_matrix (trc_red, trc_green, trc_blue,
                                         rx, ry, rz, gx, gy, gz, bx, by, bz);
      if (ret)
