@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU Lesser General
  * Public License along with this library; if not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #define MAX_SPACES   100
@@ -167,7 +167,7 @@ static void babl_matrix_equalize (double *in_mat)
   }
 }
 
-static void babl_space_compute_matrices (BablSpace *space, int equalize_matrix)
+static void babl_space_compute_matrices (BablSpace *space, BablSpaceFlags equalize_matrix)
 {
 #define _ space->
   /* transform spaces xy(Y) specified data to XYZ */
@@ -230,18 +230,50 @@ babl_space_from_rgbxyz_matrix (const char *name,
                                const Babl *trc_blue)
 {
   int i=0;
-  static BablSpace space;
+  BablSpace space;
   space.instance.class_type = BABL_SPACE;
   space.instance.id         = 0;
-  space.xr = rx;
-  space.yr = gx;
-  space.xg = bx;
-  space.yg = ry;
-  space.xb = gy;
-  space.yb = by;
-  space.xw = rz;
-  space.yw = gz;
-  space.pad = bz;
+
+  /* transplant matrixes */
+
+  space.RGBtoXYZ[0] = rx;
+  space.RGBtoXYZ[1] = gx;
+  space.RGBtoXYZ[2] = bx;
+  space.RGBtoXYZ[3] = ry;
+  space.RGBtoXYZ[4] = gy;
+  space.RGBtoXYZ[5] = by;
+  space.RGBtoXYZ[6] = rz;
+  space.RGBtoXYZ[7] = gz;
+  space.RGBtoXYZ[8] = bz;
+
+  babl_matrix_invert (space.RGBtoXYZ, space.XYZtoRGB);
+
+  babl_matrix_to_float (space.RGBtoXYZ, space.RGBtoXYZf);
+  babl_matrix_to_float (space.XYZtoRGB, space.XYZtoRGBf);
+
+  {
+    double red[3]={1.,.0,.0};
+    double xyz[3]={1.,.0,.0};
+    _babl_space_to_xyz ((Babl*)&space, &red[0], &xyz[0]);
+    space.xr = xyz[0] / (xyz[0] + xyz[1] + xyz[2]);
+    space.yr = xyz[1] / (xyz[0] + xyz[1] + xyz[2]);
+  }
+  {
+    double green[3]={0.,1.0,.0};
+    double xyz[3]={0.,1.0,.0};
+    _babl_space_to_xyz ((Babl*)&space, &green[0], &xyz[0]);
+    space.xg = xyz[0] / (xyz[0] + xyz[1] + xyz[2]);
+    space.yg = xyz[1] / (xyz[0] + xyz[1] + xyz[2]);
+  }
+  {
+    double blue[3]={0.,.0,1.0};
+    double xyz[3]={0.,1.0,.0};
+    _babl_space_to_xyz ((Babl*)&space, &blue[0], &xyz[0]);
+    space.xb = xyz[0] / (xyz[0] + xyz[1] + xyz[2]);
+    space.yb = xyz[1] / (xyz[0] + xyz[1] + xyz[2]);
+  }
+  space.xw = wx / (wx+wy+wz);
+  space.yw = wy / (wx+wy+wz);
 
   space.whitepoint[0] = wx;
   space.whitepoint[1] = wy;
@@ -266,49 +298,30 @@ babl_space_from_rgbxyz_matrix (const char *name,
     babl_log ("too many BablSpaces");
     return NULL;
   }
-  /* transplant matrixes */
-  //babl_space_compute_matrices (&space_db[i]);
-  space.RGBtoXYZ[0] = rx;
-  space.RGBtoXYZ[1] = gx;
-  space.RGBtoXYZ[2] = bx;
-  space.RGBtoXYZ[3] = ry;
-  space.RGBtoXYZ[4] = gy;
-  space.RGBtoXYZ[5] = by;
-  space.RGBtoXYZ[6] = rz;
-  space.RGBtoXYZ[7] = gz;
-  space.RGBtoXYZ[8] = bz;
-
-  babl_matrix_invert (space.RGBtoXYZ, space.XYZtoRGB);
-
-  babl_matrix_to_float (space.RGBtoXYZ, space.RGBtoXYZf);
-  babl_matrix_to_float (space.XYZtoRGB, space.XYZtoRGBf);
 
   space_db[i]=space;
   space_db[i].instance.name = space_db[i].name;
   if (name)
     snprintf (space_db[i].name, sizeof (space_db[i].name), "%s", name);
   else
-    snprintf (space_db[i].name, sizeof (space_db[i].name), "space-%.4f,%.4f_%.4f,%.4f_%.4f_%.4f,%.4f_%.4f,%.4f_%s,%s,%s",
-                       rx, gx, bx,
-                       ry, gy, by,
-                       rz, gz, bz,
-             babl_get_name (space.trc[0]),
+          /* XXX: this can get longer than 256bytes ! */
+    snprintf (space_db[i].name, sizeof (space_db[i].name),
+             "space-%.4f,%.4f_%.4f,%.4f_%.4f,%.4f_%.4f,%.4f_%s,%s,%s",
+             wx,wy,rx,ry,bx,by,gx,gy,babl_get_name (space.trc[0]),
              babl_get_name(space.trc[1]), babl_get_name(space.trc[2]));
 
-
   return (Babl*)&space_db[i];
 }
 
-const Babl *
-babl_space_from_chromaticities (const char *name,
-                                double wx, double wy,
-                                double rx, double ry,
-                                double gx, double gy,
-                                double bx, double by,
-                                const Babl *trc_red,
-                                const Babl *trc_green,
-                                const Babl *trc_blue,
-                                int equalize_matrix)
+const Babl * babl_space_from_chromaticities (const char *name,
+                                             double wx, double wy,
+                                             double rx, double ry,
+                                             double gx, double gy,
+                                             double bx, double by,
+                                             const Babl *trc_red,
+                                             const Babl *trc_green,
+                                             const Babl *trc_blue,
+                                             BablSpaceFlags flags)
 {
   int i=0;
   static BablSpace space;
@@ -358,7 +371,7 @@ babl_space_from_chromaticities (const char *name,
              babl_get_name(space.trc[1]), babl_get_name(space.trc[2]));
 
   /* compute matrixes */
-  babl_space_compute_matrices (&space_db[i], equalize_matrix);
+  babl_space_compute_matrices (&space_db[i], flags);
 
   return (Babl*)&space_db[i];
 }
@@ -538,7 +551,9 @@ static void prep_conversion (const Babl *babl)
   const Babl *source_space = babl_conversion_get_source_space (conversion);
   float *matrixf;
   int i;
-  float *lut;
+  float *lut_red;
+  float *lut_green;
+  float *lut_blue;
 
   double matrix[9];
   babl_matrix_mul_matrix (
@@ -546,15 +561,18 @@ static void prep_conversion (const Babl *babl)
      (conversion->conversion.source)->format.space->space.RGBtoXYZ,
      matrix);
 
-  matrixf = babl_calloc (sizeof (float), 9 + 256); // we leak this matrix , which is a singleton
+  matrixf = babl_calloc (sizeof (float), 9 + 256 * 3); // we leak this matrix , which is a singleton
   babl_matrix_to_float (matrix, matrixf);
   conversion->conversion.data = matrixf;
 
-  lut = matrixf + 9;
+  lut_red = matrixf + 9;
+  lut_green = lut_red + 256;
+  lut_blue = lut_green + 256;
   for (i = 0; i < 256; i++)
   {
-    lut[i] = babl_trc_to_linear (source_space->space.trc[0], i/255.0);
-    // XXX: should have green and blue luts as well
+    lut_red[i] = babl_trc_to_linear (source_space->space.trc[0], i/255.0);
+    lut_green[i] = babl_trc_to_linear (source_space->space.trc[1], i/255.0);
+    lut_blue[i] = babl_trc_to_linear (source_space->space.trc[2], i/255.0);
   }
 }
 
@@ -639,7 +657,9 @@ universal_nonlinear_rgba_u8_converter (const Babl *conversion,unsigned char *src
   const Babl *destination_space = conversion->conversion.destination->format.space;
 
   float * matrixf = data;
-  float * in_trc_lut = matrixf + 9;
+  float * in_trc_lut_red = matrixf + 9;
+  float * in_trc_lut_green  = in_trc_lut_red + 256;
+  float * in_trc_lut_blue  = in_trc_lut_green + 256;
   int i;
   uint8_t *rgba_in_u8 = (void*)src_char;
   uint8_t *rgba_out_u8 = (void*)dst_char;
@@ -648,9 +668,9 @@ universal_nonlinear_rgba_u8_converter (const Babl *conversion,unsigned char *src
 
   for (i = 0; i < samples; i++)
   {
-    rgb[i*4+0]=in_trc_lut[rgba_in_u8[i*4+0]];
-    rgb[i*4+1]=in_trc_lut[rgba_in_u8[i*4+1]];
-    rgb[i*4+2]=in_trc_lut[rgba_in_u8[i*4+2]];
+    rgb[i*4+0]=in_trc_lut_red[rgba_in_u8[i*4+0]];
+    rgb[i*4+1]=in_trc_lut_green[rgba_in_u8[i*4+1]];
+    rgb[i*4+2]=in_trc_lut_blue[rgba_in_u8[i*4+2]];
     rgba_out_u8[i*4+3] = rgba_in_u8[i*4+3];
   }
 
@@ -698,7 +718,9 @@ universal_nonlinear_rgb_u8_converter (const Babl *conversion,unsigned char *src_
   const Babl *destination_space = conversion->conversion.destination->format.space;
 
   float * matrixf = data;
-  float * in_trc_lut = matrixf + 9;
+  float * in_trc_lut_red = matrixf + 9;
+  float * in_trc_lut_green = in_trc_lut_red + 256;
+  float * in_trc_lut_blue = in_trc_lut_green + 256;
   int i;
   uint8_t *rgb_in_u8 = (void*)src_char;
   uint8_t *rgb_out_u8 = (void*)dst_char;
@@ -707,9 +729,9 @@ universal_nonlinear_rgb_u8_converter (const Babl *conversion,unsigned char *src_
 
   for (i = 0; i < samples; i++)
   {
-    rgba_out[i*4+0]=in_trc_lut[rgb_in_u8[i*3+0]];
-    rgba_out[i*4+1]=in_trc_lut[rgb_in_u8[i*3+1]];
-    rgba_out[i*4+2]=in_trc_lut[rgb_in_u8[i*3+2]];
+    rgba_out[i*4+0]=in_trc_lut_red[rgb_in_u8[i*3+0]];
+    rgba_out[i*4+1]=in_trc_lut_green[rgb_in_u8[i*3+1]];
+    rgba_out[i*4+2]=in_trc_lut_blue[rgb_in_u8[i*3+2]];
     rgba_out[i*4+3]=rgb_in_u8[i*3+2] * 255.5f;
   }
 
@@ -792,7 +814,9 @@ universal_nonlinear_rgba_u8_converter_sse2 (const Babl *conversion,unsigned char
   const Babl *destination_space = conversion->conversion.destination->format.space;
 
   float * matrixf = data;
-  float * in_trc_lut = matrixf + 9;
+  float * in_trc_lut_red = matrixf + 9;
+  float * in_trc_lut_green = in_trc_lut_red + 256;
+  float * in_trc_lut_blue = in_trc_lut_green + 256;
   int i;
   uint8_t *rgba_in_u8 = (void*)src_char;
   uint8_t *rgba_out_u8 = (void*)dst_char;
@@ -801,9 +825,9 @@ universal_nonlinear_rgba_u8_converter_sse2 (const Babl *conversion,unsigned char
 
   for (i = 0; i < samples * 4; i+= 4)
   {
-    rgba_out[i+0]=in_trc_lut[rgba_in_u8[i+0]];
-    rgba_out[i+1]=in_trc_lut[rgba_in_u8[i+1]];
-    rgba_out[i+2]=in_trc_lut[rgba_in_u8[i+2]];
+    rgba_out[i+0]=in_trc_lut_red[rgba_in_u8[i+0]];
+    rgba_out[i+1]=in_trc_lut_green[rgba_in_u8[i+1]];
+    rgba_out[i+2]=in_trc_lut_blue[rgba_in_u8[i+2]];
     rgba_out_u8[i+3] = rgba_in_u8[i+3];
   }
 
@@ -827,7 +851,9 @@ universal_nonlinear_rgb_u8_converter_sse2 (const Babl *conversion,unsigned char
   const Babl *destination_space = conversion->conversion.destination->format.space;
 
   float * matrixf = data;
-  float * in_trc_lut = matrixf + 9;
+  float * in_trc_lut_red = matrixf + 9;
+  float * in_trc_lut_green = in_trc_lut_red + 256;
+  float * in_trc_lut_blue = in_trc_lut_green + 256;
   int i;
   uint8_t *rgb_in_u8 = (void*)src_char;
   uint8_t *rgb_out_u8 = (void*)dst_char;
@@ -836,9 +862,9 @@ universal_nonlinear_rgb_u8_converter_sse2 (const Babl *conversion,unsigned char
 
   for (i = 0; i < samples; i++)
   {
-    rgba_out[i*4+0]=in_trc_lut[rgb_in_u8[i*3+0]];
-    rgba_out[i*4+1]=in_trc_lut[rgb_in_u8[i*3+1]];
-    rgba_out[i*4+2]=in_trc_lut[rgb_in_u8[i*3+2]];
+    rgba_out[i*4+0]=in_trc_lut_red[rgb_in_u8[i*3+0]];
+    rgba_out[i*4+1]=in_trc_lut_green[rgb_in_u8[i*3+1]];
+    rgba_out[i*4+2]=in_trc_lut_blue[rgb_in_u8[i*3+2]];
   }
 
   babl_matrix_mul_vectorff_buf4_sse2 (matrixf, rgba_out, rgba_out, samples);
@@ -1008,6 +1034,54 @@ const Babl *babl_space_match_trc_matrix (const Babl *trc_red,
   return NULL;
 }
 
+const Babl *babl_space_with_trc (const Babl *babl, const Babl *trc)
+{
+  double xw, yw, xr, yr, xg, yg, xb, yb;
+  const Babl *red_trc = NULL;
+  const Babl *green_trc = NULL;
+  const Babl *blue_trc = NULL;
+
+  babl_space_get (babl,
+                  &xw, &yw,
+                  &xr, &yr,
+                  &xg, &yg,
+                  &xb, &yb,
+                  &red_trc, &green_trc, &blue_trc);
+  if (red_trc == trc && green_trc == trc && blue_trc == trc)
+    return babl;
+  return babl_space_from_chromaticities (NULL,
+                                         xw, yw, xr, yr, xg, yg, xb, yb, trc, trc, trc,
+                                         BABL_SPACE_FLAG_EQUALIZE);
+
+}
+
+void babl_space_get (const Babl *babl,
+                     double *xw, double *yw,
+                     double *xr, double *yr,
+                     double *xg, double *yg,
+                     double *xb, double *yb,
+                     const Babl **red_trc,
+                     const Babl **green_trc,
+                     const Babl **blue_trc)
+{
+  const BablSpace *space = &babl->space;
+  /* XXX: note: for spaces set by matrix should be possible to derive
+                the chromaticities of r,g,b and thus even then keep this
+                is canonical data
+   */
+  if(xw)*xw = space->xw;
+  if(yw)*yw = space->yw;
+  if(xr)*xr = space->xr;
+  if(yr)*yr = space->yr;
+  if(xg)*xg = space->xg;
+  if(yg)*yg = space->yg;
+  if(xb)*xb = space->xb;
+  if(yb)*yb = space->yb;
+  if(red_trc)*red_trc = space->trc[0];
+  if(green_trc)*green_trc = space->trc[1];
+  if(blue_trc)*blue_trc = space->trc[2];
+}
+
 /* Trademarks:
  *
  * International Color Consortium is a registered trademarks of the.
