@@ -52,16 +52,9 @@ conv_rgbafloat_linear_cairo24_le (const Babl *conversion,unsigned char *src_char
 
   while (n--)
     {
-      if (src[3] < BABL_ALPHA_THRESHOLD)
-        {
-          *(int *)dst = 0;
-        }
-      else
-        {
-          dst[0] = conv_float_u8_two_table_map (src[2]);
-          dst[1] = conv_float_u8_two_table_map (src[1]);
-          dst[2] = conv_float_u8_two_table_map (src[0]);
-        }
+      dst[0] = conv_float_u8_two_table_map (src[2]);
+      dst[1] = conv_float_u8_two_table_map (src[1]);
+      dst[2] = conv_float_u8_two_table_map (src[0]);
       src += 4;
       dst += 4;
     }
@@ -96,7 +89,7 @@ conv_rgbafloat_linear_rgbu8_gamma (const Babl *conversion,unsigned char *src_cha
 
   while (n--)
     {
-      if (src[3] < BABL_ALPHA_THRESHOLD)
+      if (src[3] <= BABL_ALPHA_FLOOR)
         {
           dst[0] = 0;
           dst[1] = 0;
@@ -124,20 +117,10 @@ conv_rgbafloat_linear_rgbau8_gamma (const Babl *conversion,unsigned char *src_ch
 
   while (n--)
     {
-      if (src[3] <=0)
-        {
-          dst[0] = 0;
-          dst[1] = 0;
-          dst[2] = 0;
-          dst[3] = 0;
-        }
-      else
-        {
-          dst[0] = conv_float_u8_two_table_map (src[0]);
-          dst[1] = conv_float_u8_two_table_map (src[1]);
-          dst[2] = conv_float_u8_two_table_map (src[2]);
-          dst[3] = src[3] * 0xff + 0.5;
-        }
+      dst[0] = conv_float_u8_two_table_map (src[0]);
+      dst[1] = conv_float_u8_two_table_map (src[1]);
+      dst[2] = conv_float_u8_two_table_map (src[2]);
+      dst[3] = src[3] * 0xff + 0.5;
       src += 4;
       dst += 4;
     }
